@@ -36,12 +36,18 @@ class UsageError(Exception):
         self.msg = message
 
 
+class Messages:
+    """Notification messages emitted by these commands"""
+
+    DIFF_LOADING = object()
+
+
 class EditModel(ContextCommand):
     """Commands that mutate the main model diff data"""
 
     UNDOABLE = True
 
-    def __init__(self, context):
+    def __init__(self, context, finalizer=None):
         """Common edit operations on the main model"""
         super().__init__(context)
 
@@ -56,22 +62,33 @@ class EditModel(ContextCommand):
         self.new_mode = self.old_mode
         self.new_diff_type = self.old_diff_type
         self.new_file_type = self.old_file_type
+        self.finalizer = finalizer
 
     def do(self):
         """Perform the operation."""
+        if not super().do():
+            return
         self.model.filename = self.new_filename
         self.model.set_mode(self.new_mode)
         self.model.set_diff_text(self.new_diff_text)
         self.model.set_diff_type(self.new_diff_type)
         self.model.set_file_type(self.new_file_type)
+        if self.finalizer is not None:
+            # Finalizers inherit our timestamp so that they fire when we fire.
+            self.finalizer.timestamp = self.timestamp
+            self.context.command_bus.do(self.finalizer)
 
     def undo(self):
         """Undo the operation."""
+        if not super().undo():
+            return
         self.model.filename = self.old_filename
         self.model.set_mode(self.old_mode)
         self.model.set_diff_text(self.old_diff_text)
         self.model.set_diff_type(self.old_diff_type)
         self.model.set_file_type(self.old_file_type)
+        if self.finalizer is not None:
+            self.context.command_bus.undo(self.finalizer)
 
 
 class ConfirmAction(ContextCommand):
@@ -222,15 +239,19 @@ class AmendMode(EditModel):
         super().__init__(context)
         self.skip = False
         self.amending = amend
+        self.old_commit_author = self.model.commit_author
         self.old_commitmsg = self.model.commitmsg
         self.old_mode = self.model.mode
 
         if self.amending:
+            author, commitmsg = gitcmds.prev_author_and_commitmsg(context)
+            self.new_author = author
+            self.new_commitmsg = commitmsg
             self.new_mode = self.model.mode_amend
-            self.new_commitmsg = gitcmds.prev_commitmsg(context)
             AmendMode.LAST_MESSAGE = self.model.commitmsg
             return
         # else, amend unchecked, regular commit
+        self.new_author = ''
         self.new_mode = self.model.mode_none
         self.new_diff_text = ''
         self.new_commitmsg = self.model.commitmsg
@@ -258,6 +279,7 @@ class AmendMode(EditModel):
                 return
         self.skip = False
         super().do()
+        self.model.set_commit_author(self.new_author)
         self.model.set_commitmsg(self.new_commitmsg)
         self.model.update_file_status()
         self.context.selection.reset(emit=True)
@@ -265,6 +287,7 @@ class AmendMode(EditModel):
     def undo(self):
         if self.skip:
             return
+        self.model.set_commit_author(self.old_commit_author)
         self.model.set_commitmsg(self.old_commitmsg)
         super().undo()
         self.model.update_file_status()
@@ -799,7 +822,9 @@ class UndoLastCommit(ResetCommand):
 class Commit(ResetMode):
     """Attempt to create a new commit."""
 
-    def __init__(self, context, amend, msg, sign, no_verify=False):
+    def __init__(
+        self, context, amend, msg, sign, no_verify=False, author=None, date=None
+    ):
         super().__init__(context)
         self.amend = amend
         self.msg = msg
@@ -807,21 +832,39 @@ class Commit(ResetMode):
         self.no_verify = no_verify
         self.old_commitmsg = self.model.commitmsg
         self.new_commitmsg = ''
+        self.author = author
+        self.date = date
 
     def do(self):
         # Create the commit message file
         context = self.context
         msg = self.msg
         tmp_file = utils.tmp_filename('commit-message')
+        add_env = {
+            'NO_COLOR': '1',
+            'TERM': 'dumb',
+        }
+        add_env.update(main.autodetect_proxy_environ())
+        kwargs = {}
+        # Override the commit date.
+        if self.date:
+            add_env['GIT_AUTHOR_DATE'] = self.date
+            add_env['GIT_COMMITTER_DATE'] = self.date
+            kwargs['date'] = self.date
+        # Override the commit author.
+        if self.author:
+            kwargs['author'] = self.author
         try:
             core.write(tmp_file, msg)
             # Run 'git commit'
             status, out, err = self.git.commit(
+                _add_env=add_env,
                 F=tmp_file,
                 v=True,
                 gpg_sign=self.sign,
                 amend=self.amend,
                 no_verify=self.no_verify,
+                **kwargs,
             )
         finally:
             core.unlink(tmp_file)
@@ -869,7 +912,7 @@ class Ignore(ContextCommand):
         new_additions = '\n'.join(self.filenames) + '\n'
         for_status = new_additions
         if self.local:
-            filename = os.path.join('.git', 'info', 'exclude')
+            filename = self.git.git_path('info', 'exclude')
         else:
             filename = '.gitignore'
         if core.exists(filename):
@@ -1187,7 +1230,10 @@ class DeleteRemoteBranch(DeleteBranch):
         self.remote = remote
 
     def action(self):
-        return self.git.push(self.remote, self.branch, delete=True)
+        kwargs = {}
+        main.autodetect_proxy(self.context, kwargs)
+        main.no_color(kwargs)
+        return self.git.push(self.remote, self.branch, delete=True, **kwargs)
 
     def success(self):
         self.model.update_status()
@@ -1236,13 +1282,29 @@ class DiffAgainstCommitMode(ContextCommand):
         self.model.update_file_status()
 
 
-class DiffText(EditModel):
+class DiffText(ContextCommand):
     """Set the diff type to text"""
 
     def __init__(self, context):
         super().__init__(context)
         self.new_file_type = main.Types.TEXT
         self.new_diff_type = main.Types.TEXT
+        self.old_file_type = self.model.file_type
+        self.old_diff_type = self.model.diff_type
+
+    def do(self):
+        """Update the diff and file type"""
+        if not super().do():
+            return
+        self.model.set_diff_type(self.new_diff_type)
+        self.model.set_file_type(self.new_file_type)
+
+    def undo(self):
+        """Revert the updating of diff and file types"""
+        if not super().undo():
+            return
+        self.model.set_diff_type(self.old_diff_type)
+        self.model.set_file_type(self.old_file_type)
 
 
 class ToggleDiffType(ContextCommand):
@@ -1458,11 +1520,19 @@ class DiffImage(EditModel):
         return images
 
 
+class DiffLoading(ContextCommand):
+    """Notify the diff viewer the a diff is loading"""
+
+    def do(self):
+        self.context.notifier.notify(Messages.DIFF_LOADING)
+
+
 class Diff(EditModel):
     """Perform a diff and set the model's current text."""
 
-    def __init__(self, context, filename, cached=False, deleted=False):
-        super().__init__(context)
+    def __init__(self, context, filename, cached=False, deleted=False, finalizer=None):
+        DiffLoading(context).do()
+        super().__init__(context, finalizer=finalizer)
         opts = {}
         if cached and gitcmds.is_valid_ref(context, self.model.head):
             opts['ref'] = self.model.head
@@ -1477,6 +1547,7 @@ class Diffstat(EditModel):
     """Perform a diffstat and set the model's diff text."""
 
     def __init__(self, context):
+        DiffLoading(context).do()
         super().__init__(context)
         cfg = self.cfg
         diff_context = cfg.get('diff.context', 3)
@@ -1497,13 +1568,16 @@ class Diffstat(EditModel):
 class DiffStaged(Diff):
     """Perform a staged diff on a file."""
 
-    def __init__(self, context, filename, deleted=None):
-        super().__init__(context, filename, cached=True, deleted=deleted)
+    def __init__(self, context, filename, deleted=None, finalizer=None):
+        super().__init__(
+            context, filename, cached=True, deleted=deleted, finalizer=finalizer
+        )
         self.new_mode = self.model.mode_index
 
 
 class DiffStagedSummary(EditModel):
     def __init__(self, context):
+        DiffLoading(context).do()
         super().__init__(context)
         diff = self.git.diff(
             self.model.head,
@@ -1526,9 +1600,19 @@ class Edit(ContextCommand):
     def name():
         return N_('Launch Editor')
 
-    def __init__(self, context, filenames, line_number=None, background_editor=False):
+    def __init__(
+        self,
+        context,
+        filenames,
+        selected_filename=None,
+        line_number=None,
+        background_editor=False,
+    ):
         super().__init__(context)
+        if filenames and not selected_filename:
+            selected_filename = filenames[0]
         self.filenames = filenames
+        self.selected_filename = selected_filename
         self.line_number = line_number
         self.background_editor = background_editor
 
@@ -1536,38 +1620,47 @@ class Edit(ContextCommand):
         context = self.context
         if not self.filenames:
             return
-        filename = self.filenames[0]
-        if not core.exists(filename):
-            return
+
         if self.background_editor:
             editor = prefs.background_editor(context)
         else:
             editor = prefs.editor(context)
-        opts = []
-
-        if self.line_number is None:
-            opts = self.filenames
+        if self.line_number is None or self.selected_filename is None:
+            args = self.filenames
         else:
-            # Single-file w/ line-numbers (likely from grep)
+            args = []
+            # grep and diff are able to open files at specific line numbers.
+            # We only know the line number for the first file.
+            # Some editors can only apply the '+<line-number>' argument to one file.
+            filename = self.selected_filename
             editor_opts = {
-                '*vim*': [filename, '+%s' % self.line_number],
+                '*vim*': ['+%s' % self.line_number, filename],
                 '*emacs*': ['+%s' % self.line_number, filename],
                 '*textpad*': [f'{filename}({self.line_number},0)'],
                 '*notepad++*': ['-n%s' % self.line_number, filename],
                 '*subl*': [f'{filename}:{self.line_number}'],
+                'code': [f'--goto {filename}:{self.line_number}'],
+                'cursor': [f'--goto {filename}:{self.line_number}'],
             }
 
-            opts = self.filenames
-            for pattern, opt in editor_opts.items():
-                if fnmatch(editor, pattern):
-                    opts = opt
+            use_line_numbers = False
+            for pattern, opts in editor_opts.items():
+                if fnmatch(editor, pattern) or fnmatch(
+                    os.path.basename(editor), pattern
+                ):
+                    args.extend(opts)
+                    use_line_numbers = True
                     break
+            if use_line_numbers:
+                args.extend(fname for fname in self.filenames if fname != filename)
+            else:
+                args = self.filenames
 
         try:
-            core.fork(utils.shell_split(editor) + opts)
-        except (OSError, ValueError) as e:
+            core.fork(utils.shell_split(editor) + args)
+        except (OSError, ValueError) as err:
             message = N_('Cannot exec "%s": please configure your editor') % editor
-            _, details = utils.format_exception(e)
+            _, details = utils.format_exception(err)
             Interaction.critical(N_('Error Editing File'), message, details)
 
 
@@ -1640,6 +1733,11 @@ class LaunchEditorAtLine(LaunchEditor):
     def __init__(self, context):
         super().__init__(context)
         self.line_number = context.selection.line_number
+        # Ensure that the model's filename is present in self.filenames otherwise we
+        # will open a file that the user never requested. This constraint also ensures
+        # that the line number corresponds to the selected filename.
+        if context.model.filename in self.filenames:
+            self.selected_filename = context.model.filename
 
 
 class LoadCommitMessageFromFile(ContextCommand):
@@ -1781,6 +1879,18 @@ class Merge(ContextCommand):
         return status, out, err
 
 
+class MergeBranch(Merge):
+    """Merge a branch with default settings applied"""
+
+    def __init__(self, context, branch):
+        values = context.settings.get('merge')
+        no_commit = not values.get('commit', True)
+        squash = values.get('squash', False)
+        no_ff = values.get('no-ff', False)
+        sign = values.get('sign', False)
+        super().__init__(context, branch, no_commit, squash, no_ff, sign)
+
+
 class OpenDefaultApp(ContextCommand):
     """Open a file using the OS default."""
 
@@ -1956,6 +2066,16 @@ class NewBareRepo(ContextCommand):
         return status == 0
 
 
+class NoOp(ContextCommand):
+    """A command that does nothing"""
+
+    def __init__(self, context, *args, **kwargs):
+        super().__init__(context)
+
+    def do(self):
+        pass
+
+
 def unix_path(path, is_win32=utils.is_win32):
     """Git for Windows requires Unix paths, so force them here"""
     if is_win32():
@@ -2028,8 +2148,13 @@ class Rebase(ContextCommand):
         kwargs.update(self.kwargs)
 
         # Prompt to determine whether or not to use "git rebase --update-refs".
+        update_refs = self.context.cfg.get(prefs.REBASE_UPDATE_REFS)
         has_update_refs = version.check_git(self.context, 'rebase-update-refs')
-        if has_update_refs and not kwargs.get('update_refs', False):
+        if (
+            has_update_refs
+            and update_refs is None
+            and not kwargs.get('update_refs', False)
+        ):
             title = N_('Update stacked branches when rebasing?')
             text = N_(
                 '"git rebase --update-refs" automatically force-updates any\n'
@@ -2173,6 +2298,7 @@ class Refresh(ContextCommand):
         self.model.update_status(update_index=True)
         self.cfg.update()
         self.fsmonitor.refresh()
+        self.selection.selection_changed.emit()
 
 
 class RefreshConfig(ContextCommand):
@@ -2414,8 +2540,8 @@ def format_hex(data):
 class ShowUntracked(EditModel):
     """Show an untracked file."""
 
-    def __init__(self, context, filename):
-        super().__init__(context)
+    def __init__(self, context, filename, finalizer=None):
+        super().__init__(context, finalizer=finalizer)
         self.new_filename = filename
         if gitcmds.is_binary(context, filename):
             self.new_mode = self.model.mode_untracked
