@@ -1,3 +1,5 @@
+import sys
+
 from qtpy.QtGui import QKeySequence
 from qtpy.QtCore import Qt
 
@@ -13,8 +15,8 @@ BLAME = hotkey(Qt.CTRL | Qt.SHIFT | Qt.Key_B)
 BRANCH = hotkey(Qt.CTRL | Qt.Key_B)
 CHECKOUT = hotkey(Qt.ALT | Qt.Key_B)
 CHERRY_PICK = hotkey(Qt.CTRL | Qt.SHIFT | Qt.Key_C)
+COPY_COMMIT_ID = hotkey(Qt.CTRL | Qt.ALT | Qt.Key_C)
 COPY_DIFF = hotkey(Qt.ALT | Qt.SHIFT | Qt.Key_C)
-COPY_SHA1 = hotkey(Qt.CTRL | Qt.ALT | Qt.Key_C)
 DIFFSTAT = hotkey(Qt.ALT | Qt.Key_D)
 DIFF = hotkey(Qt.CTRL | Qt.Key_D)
 DIFF_SECONDARY = hotkey(Qt.CTRL | Qt.SHIFT | Qt.Key_D)
@@ -48,7 +50,13 @@ FOCUS_DIFF = hotkey(Qt.CTRL | Qt.Key_J)
 FOCUS_INPUT = hotkey(Qt.CTRL | Qt.Key_L)
 FOCUS_STATUS = hotkey(Qt.CTRL | Qt.Key_K)
 FOCUS_TREE = hotkey(Qt.CTRL | Qt.Key_K)
-AMEND = hotkey(Qt.CTRL | Qt.Key_M)
+AMEND_DEFAULT = hotkey(Qt.CTRL | Qt.Key_M)
+AMEND_MACOS = hotkey(Qt.ALT | Qt.Key_M)
+if sys.platform == 'darwin':
+    AMEND = (AMEND_MACOS,)
+else:
+    AMEND = (AMEND_DEFAULT, AMEND_MACOS)
+MACOS_MINIMIZE = hotkey(Qt.CTRL | Qt.Key_M)
 MERGE = hotkey(Qt.CTRL | Qt.SHIFT | Qt.Key_M)
 PUSH = hotkey(Qt.CTRL | Qt.Key_P)
 PULL = hotkey(Qt.CTRL | Qt.SHIFT | Qt.Key_P)
@@ -73,6 +81,7 @@ TERMINAL = hotkey(Qt.CTRL | Qt.SHIFT | Qt.Key_T)
 STAGE_UNTRACKED = hotkey(Qt.ALT | Qt.Key_U)
 REVERT = hotkey(Qt.CTRL | Qt.Key_U)
 REVERT_ALT = hotkey(Qt.ALT | Qt.SHIFT | Qt.Key_R)
+REVERT_UNSTAGED_EDITS = hotkey(Qt.ALT | Qt.CTRL | Qt.Key_U)
 EDIT_AND_REVERT = hotkey(Qt.CTRL | Qt.SHIFT | Qt.Key_U)
 WORD_RIGHT = hotkey(Qt.Key_W)
 
@@ -106,6 +115,8 @@ UP = hotkey(Qt.Key_Up)
 
 CTRL_RETURN = hotkey(Qt.CTRL | Qt.Key_Return)
 CTRL_ENTER = hotkey(Qt.CTRL | Qt.Key_Enter)
+CTRL_LEFT = hotkey(Qt.CTRL | Qt.Key_Left)
+CTRL_RIGHT = hotkey(Qt.CTRL | Qt.Key_Right)
 
 # Rebase
 REBASE_START_AND_CONTINUE = hotkey(Qt.ALT | Qt.Key_R)
@@ -114,6 +125,7 @@ REBASE_REWORD = (hotkey(Qt.Key_2), hotkey(Qt.Key_R))
 REBASE_EDIT = (hotkey(Qt.Key_3), hotkey(Qt.Key_E))
 REBASE_FIXUP = (hotkey(Qt.Key_4), hotkey(Qt.Key_F))
 REBASE_SQUASH = (hotkey(Qt.Key_5), hotkey(Qt.Key_S))
+REBASE_DROP = (hotkey(Qt.Key_6), hotkey(Qt.Key_D))
 
 UNDO = hotkey(Qt.CTRL | Qt.Key_Z)
 REDO = hotkey(Qt.SHIFT | Qt.CTRL | Qt.Key_Z)
