@@ -37,7 +37,7 @@ def read_json(path):
         return {}
 
 
-def write_json(values, path):
+def write_json(values, path, sync=True):
     """Write the specified values dict to a JSON file at the specified path"""
     try:
         parent = os.path.dirname(path)
@@ -45,6 +45,8 @@ def write_json(values, path):
             core.makedirs(parent)
         with core.open_write(path) as fp:
             json.dump(values, fp, indent=4)
+            if sync:
+                core.fsync(fp.fileno())
     except (ValueError, TypeError, OSError):
         sys.stderr.write('git-cola: error writing "%s"\n' % path)
         return False
@@ -165,7 +167,7 @@ class Settings:
     def path(self):
         return self.config_path
 
-    def save(self):
+    def save(self, sync=True):
         """Write settings robustly to avoid losing data during a forced shutdown.
 
         To save robustly we take these steps:
@@ -181,7 +183,7 @@ class Settings:
         path_tmp = path + '.tmp'
         path_bak = path + '.bak'
         # Write the new settings to the .tmp file.
-        if not write_json(self.values, path_tmp):
+        if not write_json(self.values, path_tmp, sync=sync):
             return
         # Rename the current settings to a .bak file.
         if core.exists(path) and not rename_path(path, path_bak):
@@ -189,8 +191,6 @@ class Settings:
         # Rename the new settings from .tmp to the settings file.
         if not rename_path(path_tmp, path):
             return
-        # Flush the data to disk.
-        core.sync()
         # Delete the .bak file.
         if core.exists(path_bak):
             remove_path(path_bak)
@@ -207,7 +207,8 @@ class Settings:
         final last-ditch attempt to recover settings.
 
         """
-        path = self.path()
+        if path is None:
+            path = self.path()
         path_bak = path + '.bak'
         path_tmp = path + '.tmp'
 
@@ -257,17 +258,18 @@ class Settings:
         if not path:
             path = self.path()
         if core.exists(path):
-            return read_json(path)
-        # We couldn't find ~/.config/git-cola, try ~/.cola
-        values = {}
-        path = os.path.join(core.expanduser('~'), '.cola')
-        if core.exists(path):
-            json_values = read_json(path)
-            for key in self.values:
-                try:
-                    values[key] = json_values[key]
-                except KeyError:
-                    pass
+            values = read_json(path)
+        else:
+            # We couldn't find ~/.config/git-cola, try ~/.cola
+            values = {}
+            path = os.path.join(core.expanduser('~'), '.cola')
+            if core.exists(path):
+                json_values = read_json(path)
+                for key in self.values:
+                    try:
+                        values[key] = json_values[key]
+                    except KeyError:
+                        pass
         # Ensure that all stored bookmarks use normalized paths ("/" only).
         normalize = display.normalize_path
         for entry in values.get('bookmarks', []):
@@ -276,24 +278,35 @@ class Settings:
             entry['path'] = normalize(entry['path'])
         return values
 
-    def save_gui_state(self, gui):
-        """Saves settings for a cola view"""
+    def save_gui_state(self, gui, sync=True):
+        """Saves settings for a widget"""
         name = gui.name()
         self.gui_state[name] = mkdict(gui.export_state())
-        self.save()
+        self.save(sync=sync)
 
     def get_gui_state(self, gui):
-        """Returns the saved state for a gui"""
-        return self.get_gui_state_by_name(gui.name())
+        """Returns the saved state for a tool"""
+        return self.get(gui.name())
 
-    def get_gui_state_by_name(self, gui_name):
-        """Returns the saved state for a gui by name"""
+    def get(self, gui_name):
+        """Returns the saved state for a tool by name"""
         try:
             state = mkdict(self.gui_state[gui_name])
         except KeyError:
             state = self.gui_state[gui_name] = {}
         return state
 
+    def get_value(self, name, key, default=None):
+        """Return a specific setting value for the specified tool and setting key"""
+        return self.get(name).get(key, default)
+
+    def set_value(self, name, key, value, save=True, sync=True):
+        """Store a specific setting value for the specified tool and setting key value"""
+        values = self.get(name)
+        values[key] = value
+        if save:
+            self.save(sync=sync)
+
 
 def rename_entry(entries, path, name, new_name):
     normalize = display.normalize_path
