@@ -1,18 +1,20 @@
 import platform
 import webbrowser
+import os
 import sys
 
 import qtpy
+from qtpy import QtCore
 from qtpy.QtCore import Qt
 from qtpy import QtGui
 from qtpy import QtWidgets
 
 from ..i18n import N_
-from .. import core
 from .. import resources
 from .. import hotkeys
 from .. import icons
 from .. import qtutils
+from .. import utils
 from .. import version
 from . import defs
 
@@ -65,14 +67,10 @@ class AboutView(QtWidgets.QDialog):
         self.setWindowModality(Qt.WindowModal)
 
         # Top-most large icon
-        logo_pixmap = icons.cola().pixmap(defs.huge_icon, defs.large_icon)
-
-        self.logo_label = QtWidgets.QLabel()
-        self.logo_label.setPixmap(logo_pixmap)
+        self.logo_label = qtutils.pixmap_label(icons.cola(), defs.huge_icon)
         self.logo_label.setAlignment(Qt.AlignCenter)
 
-        self.logo_text_label = QtWidgets.QLabel()
-        self.logo_text_label.setText('Git Cola')
+        self.logo_text_label = qtutils.label(text='Git Cola')
         self.logo_text_label.setAlignment(Qt.AlignLeft | Qt.AlignCenter)
 
         font = self.logo_text_label.font()
@@ -251,115 +249,147 @@ def authors_text():
             email=mailto('davvid@gmail.com', contact, palette),
         ),
         dict(name='Daniel Harding', title=N_('Developer')),
+        dict(name='Efimov Vasily', title=N_('Developer')),
         dict(
             name='Ｖ字龍(Vdragon)',
             title=N_('Developer'),
             email=mailto('Vdragon.Taiwan@gmail.com', contact, palette),
         ),
-        dict(name='Efimov Vasily', title=N_('Developer')),
+        dict(name='Kurt McKee', title=N_('Developer')),
         dict(name='Guillaume de Bure', title=N_('Developer')),
-        dict(name='Uri Okrent', title=N_('Developer')),
         dict(name='Javier Rodriguez Cuevas', title=N_('Developer')),
+        dict(name='Uri Okrent', title=N_('Developer')),
         dict(name='Alex Chernetz', title=N_('Developer')),
         dict(name='xhl', title=N_('Developer')),
-        dict(name='Andreas Sommer', title=N_('Developer')),
-        dict(name='Thomas Kluyver', title=N_('Developer')),
-        dict(name='Minarto Margoliono', title=N_('Developer')),
         dict(name='Ville Skyttä', title=N_('Developer')),
+        dict(name='Thomas Kluyver', title=N_('Developer')),
+        dict(name='Andreas Sommer', title=N_('Developer')),
+        dict(name='nakanoi', title=N_('Developer')),
         dict(name='Szymon Judasz', title=N_('Developer')),
-        dict(name='jm4R', title=N_('Developer')),
+        dict(name='Minarto Margoliono', title=N_('Developer')),
         dict(name='Stanislaw Halik', title=N_('Developer')),
+        dict(name='jm4R', title=N_('Developer')),
         dict(name='Igor Galarraga', title=N_('Developer')),
+        dict(name='Luke Horwell', title=N_('Developer')),
         dict(name='Virgil Dupras', title=N_('Developer')),
-        dict(name='wsdfhjxc', title=N_('Developer')),
         dict(name='Barry Roberts', title=N_('Developer')),
-        dict(name='林博仁(Buo-ren Lin)', title=N_('Developer')),
+        dict(name='wsdfhjxc', title=N_('Developer')),
         dict(name='Guo Yunhe', title=N_('Developer')),
+        dict(name='malpas', title=N_('Developer')),
+        dict(name='林博仁(Buo-ren Lin)', title=N_('Developer')),
+        dict(name='Matthias Mailänder', title=N_('Developer')),
         dict(name='cclauss', title=N_('Developer')),
+        dict(name='Benjamin Somers', title=N_('Developer')),
+        dict(name='Max Harmathy', title=N_('Developer')),
         dict(name='Stefan Naewe', title=N_('Developer')),
         dict(name='Victor Nepveu', title=N_('Developer')),
-        dict(name='Pavel Rehak', title=N_('Developer')),
         dict(name='Benedict Lee', title=N_('Developer')),
-        dict(name='Tim Brown', title=N_('Developer')),
-        dict(name='Steffen Prohaska', title=N_('Developer')),
         dict(name='Filip Danilović', title=N_('Developer')),
+        dict(name='Nanda Lopes', title=N_('Developer')),
         dict(name='NotSqrt', title=N_('Developer')),
+        dict(name='Pavel Rehak', title=N_('Developer')),
+        dict(name='Steffen Prohaska', title=N_('Developer')),
+        dict(name='Thomas Kiley', title=N_('Developer')),
+        dict(name='Tim Brown', title=N_('Developer')),
+        dict(name='Chris Stefano', title=N_('Developer')),
+        dict(name='Floris Lambrechts', title=N_('Developer')),
+        dict(name='Martin Gysel', title=N_('Developer')),
         dict(name='Michael Geddes', title=N_('Developer')),
         dict(name='Rustam Safin', title=N_('Developer')),
-        dict(name='Justin Lecher', title=N_('Developer')),
+        dict(name='abid1998', title=N_('Developer')),
         dict(name='Alex Gulyás', title=N_('Developer')),
         dict(name='David Martínez Martí', title=N_('Developer')),
         dict(name='Hualiang Xie', title=N_('Developer')),
+        dict(name='Justin Lecher', title=N_('Developer')),
         dict(name='Kai Krakow', title=N_('Developer')),
         dict(name='Karl Bielefeldt', title=N_('Developer')),
         dict(name='Marco Costalba', title=N_('Developer')),
+        dict(name='Michael Baumgartner', title=N_('Developer')),
         dict(name='Michael Homer', title=N_('Developer')),
-        dict(name='Sebastian Schuberth', title=N_('Developer')),
+        dict(name='Mithil Poojary', title=N_('Developer')),
         dict(name='Sven Claussner', title=N_('Developer')),
         dict(name='Victor Gambier', title=N_('Developer')),
         dict(name='bsomers', title=N_('Developer')),
-        dict(name='real', title=N_('Developer')),
+        dict(name='mmargoliono', title=N_('Developer')),
         dict(name='v.paritskiy', title=N_('Developer')),
         dict(name='vanderkoort', title=N_('Developer')),
         dict(name='wm4', title=N_('Developer')),
+        dict(name='0xflotus', title=N_('Developer')),
+        dict(name='AJ Bagwell', title=N_('Developer')),
+        dict(name='Adrien be', title=N_('Developer')),
+        dict(name='Alexander Preißner', title=N_('Developer')),
+        dict(name='Andrej', title=N_('Developer')),
+        dict(name='Arthur Coelho', title=N_('Developer')),
         dict(name='Audrius Karabanovas', title=N_('Developer')),
-        dict(name='Matthew E. Levine', title=N_('Developer')),
-        dict(name='Matthias Mailänder', title=N_('Developer')),
-        dict(name='Md. Mahbub Alam', title=N_('Developer')),
-        dict(name='Jakub Szymański', title=N_('Developer')),
-        dict(name='ochristi', title=N_('Developer')),
-        dict(name='Miguel Boekhold', title=N_('Developer')),
-        dict(name='MiguelBoekhold', title=N_('Developer')),
-        dict(name='Mikhail Terekhov', title=N_('Developer')),
-        dict(name='Jake Biesinger', title=N_('Developer')),
+        dict(name='Axel Heider', title=N_('Developer')),
+        dict(name='Barrett Lowe', title=N_('Developer')),
+        dict(name='Ben Boeckel', title=N_('Developer')),
+        dict(name='Bob van der Linden', title=N_('Developer')),
+        dict(name='Boerje Sewing', title=N_('Developer')),
+        dict(name='Boris W', title=N_('Developer')),
+        dict(name='Bruno Cabral', title=N_('Developer')),
+        dict(name='Charles', title=N_('Developer')),
+        dict(name='Christoph Erhardt', title=N_('Developer')),
+        dict(name='Clément Pit--Claudel', title=N_('Developer')),
+        dict(name='Daniel Haskin', title=N_('Developer')),
+        dict(name='Daniel King', title=N_('Developer')),
+        dict(name='Daniel Pavel', title=N_('Developer')),
+        dict(name='DasaniT', title=N_('Developer')),
+        dict(name='Dave Cottlehuber', title=N_('Developer')),
+        dict(name='David Schwörer', title=N_('Developer')),
+        dict(name='David Zumbrunnen', title=N_('Developer')),
+        dict(name='George Vasilakos', title=N_('Developer')),
+        dict(name='Ilya Tumaykin', title=N_('Developer')),
         dict(name='Iulian Udrea', title=N_('Developer')),
+        dict(name='Jake Biesinger', title=N_('Developer')),
+        dict(name='Jakub Szymański', title=N_('Developer')),
+        dict(name='Jamie Pate', title=N_('Developer')),
+        dict(name='Jean-Francois Dagenais', title=N_('Developer')),
+        dict(name='Joachim Lusiardi', title=N_('Developer')),
+        dict(name='Karthik Manamcheri', title=N_('Developer')),
+        dict(name='Kelvie Wong', title=N_('Developer')),
+        dict(name='Klaas Neirinck', title=N_('Developer')),
+        dict(name='Kyle', title=N_('Developer')),
+        dict(name='Laszlo Boszormenyi (GCS)', title=N_('Developer')),
+        dict(name='Maciej Filipiak', title=N_('Developer')),
+        dict(name='Maicon D. Filippsen', title=N_('Developer')),
+        dict(name='Markus Heidelberg', title=N_('Developer')),
+        dict(name='Matthew E. Levine', title=N_('Developer')),
+        dict(name='Md. Mahbub Alam', title=N_('Developer')),
+        dict(name='Mikhail Terekhov', title=N_('Developer')),
+        dict(name='Niel Buys', title=N_('Developer')),
+        dict(name='Ori shalhon', title=N_('Developer')),
         dict(name='Paul Hildebrandt', title=N_('Developer')),
         dict(name='Paul Weingardt', title=N_('Developer')),
         dict(name='Paulo Fidalgo', title=N_('Developer')),
-        dict(name='Ilya Tumaykin', title=N_('Developer')),
         dict(name='Petr Gladkikh', title=N_('Developer')),
         dict(name='Philip Stark', title=N_('Developer')),
         dict(name='Radek Postołowicz', title=N_('Developer')),
         dict(name='Rainer Müller', title=N_('Developer')),
         dict(name='Ricardo J. Barberis', title=N_('Developer')),
         dict(name='Rolando Espinoza', title=N_('Developer')),
-        dict(name='George Vasilakos', title=N_('Developer')),
         dict(name="Samsul Ma'arif", title=N_('Developer')),
         dict(name='Sebastian Brass', title=N_('Developer')),
-        dict(name='Arthur Coelho', title=N_('Developer')),
+        dict(name='Sergei Dyshel', title=N_('Developer')),
         dict(name='Simon Peeters', title=N_('Developer')),
-        dict(name='Felipe Morales', title=N_('Developer')),
-        dict(name='David Zumbrunnen', title=N_('Developer')),
-        dict(name='David Schwörer', title=N_('Developer')),
         dict(name='Stephen', title=N_('Developer')),
-        dict(name='Andrej', title=N_('Developer')),
-        dict(name='Daniel Pavel', title=N_('Developer')),
-        dict(name='Daniel King', title=N_('Developer')),
-        dict(name='Daniel Haskin', title=N_('Developer')),
-        dict(name='Clément Pit--Claudel', title=N_('Developer')),
+        dict(name='Tim Gates', title=N_('Developer')),
         dict(name='Vaibhav Sagar', title=N_('Developer')),
         dict(name='Ved Vyas', title=N_('Developer')),
-        dict(name='Adrien be', title=N_('Developer')),
-        dict(name='Charles', title=N_('Developer')),
-        dict(name='Boris W', title=N_('Developer')),
-        dict(name='Ben Boeckel', title=N_('Developer')),
+        dict(name='VishnuSanal', title=N_('Developer')),
         dict(name='Voicu Hodrea', title=N_('Developer')),
+        dict(name='WNguyen14', title=N_('Developer')),
         dict(name='Wesley Wong', title=N_('Developer')),
         dict(name='Wolfgang Ocker', title=N_('Developer')),
         dict(name='Zhang Han', title=N_('Developer')),
         dict(name='beauxq', title=N_('Developer')),
-        dict(name='Jamie Pate', title=N_('Developer')),
-        dict(name='Jean-Francois Dagenais', title=N_('Developer')),
-        dict(name='Joachim Lusiardi', title=N_('Developer')),
-        dict(name='0xflotus', title=N_('Developer')),
-        dict(name='AJ Bagwell', title=N_('Developer')),
-        dict(name='Barrett Lowe', title=N_('Developer')),
-        dict(name='Karthik Manamcheri', title=N_('Developer')),
-        dict(name='Kelvie Wong', title=N_('Developer')),
-        dict(name='Kyle', title=N_('Developer')),
-        dict(name='Maciej Filipiak', title=N_('Developer')),
-        dict(name='Maicon D. Filippsen', title=N_('Developer')),
-        dict(name='Markus Heidelberg', title=N_('Developer')),
+        dict(name='bensmrs', title=N_('Developer')),
+        dict(name='lcjh', title=N_('Developer')),
+        dict(name='lefairy', title=N_('Developer')),
+        dict(name='melkecelioglu', title=N_('Developer')),
+        dict(name='ochristi', title=N_('Developer')),
+        dict(name='yael levi', title=N_('Developer')),
+        dict(name='Łukasz Wojniłowicz', title=N_('Developer')),
     )
     bug_url = 'https://github.com/git-cola/git-cola/issues'
     bug_link = qtutils.link(bug_url, bug_url)
@@ -384,29 +414,31 @@ def translators_text():
 
     translators = (
         # See the `generate-about` script in the "todo" branch.
-        # vim :read! ./Meta/generate-about --translators
+        # vim :read! ./todo/generate-about --translators
         dict(
             name='Ｖ字龍(Vdragon)',
             title=N_('Traditional Chinese (Taiwan) translation'),
             email=mailto('Vdragon.Taiwan@gmail.com', contact, palette),
         ),
         dict(name='Pavel Rehak', title=N_('Czech translation')),
-        dict(name='Zhang Han', title=N_('Simplified Chinese translation')),
         dict(name='Victorhck', title=N_('Spanish translation')),
         dict(name='Vitor Lobo', title=N_('Brazilian translation')),
-        dict(name='Igor Kopach', title=N_('Ukranian translation')),
+        dict(name='Zhang Han', title=N_('Simplified Chinese translation')),
         dict(name='Łukasz Wojniłowicz', title=N_('Polish translation')),
-        dict(name='Rafael Nascimento', title=N_('Brazilian translation')),
+        dict(name='Igor Kopach', title=N_('Ukrainian translation')),
+        dict(name='Gyuris Gellért', title=N_('Hungarian translation')),
+        dict(name='fu7mu4', title=N_('Japanese translation')),
         dict(name='Barış ÇELİK', title=N_('Turkish translation')),
+        dict(name='Guo Yunhe', title=N_('Simplified Chinese translation')),
+        dict(name='Luke Horwell', title=N_('Translation')),
         dict(name='Minarto Margoliono', title=N_('Indonesian translation')),
-        dict(name='Sven Claussner', title=N_('German translation')),
+        dict(name='Rafael Nascimento', title=N_('Brazilian translation')),
+        dict(name='Rafael Reuber', title=N_('Brazilian translation')),
         dict(name='Shun Sakai', title=N_('Japanese translation')),
+        dict(name='Sven Claussner', title=N_('German translation')),
         dict(name='Vaiz', title=N_('Russian translation')),
         dict(name='adlgrbz', title=N_('Turkish translation')),
-        dict(name='fu7mu4', title=N_('Japanese translation')),
-        dict(name='Guo Yunhe', title=N_('Simplified Chinese translation')),
-        dict(name="Samsul Ma'arif", title=N_('Indonesian translation')),
-        dict(name='Gyuris Gellért', title=N_('Hungarian translation')),
+        dict(name='Balázs Meskó', title=N_('Translation')),
         dict(name='Joachim Lusiardi', title=N_('German translation')),
         dict(name='Kai Krakow', title=N_('German translation')),
         dict(name='Louis Rousseau', title=N_('French translation')),
@@ -416,9 +448,9 @@ def translators_text():
             title=N_('Traditional Chinese (Taiwan) translation'),
         ),
         dict(name='Pilar Molina Lopez', title=N_('Spanish translation')),
-        dict(name='Rafael Reuber', title=N_('Brazilian translation')),
         dict(name='Sabri Ünal', title=N_('Turkish translation')),
-        dict(name='Balázs Meskó', title=N_('Translation')),
+        dict(name="Samsul Ma'arif", title=N_('Indonesian translation')),
+        dict(name='YAMAMOTO Kenyu', title=N_('Translation')),
         dict(name='Zeioth', title=N_('Spanish translation')),
         dict(name='balping', title=N_('Hungarian translation')),
         dict(name='p-bo', title=N_('Czech translation')),
@@ -465,23 +497,27 @@ def translators_text():
 
 
 def show_shortcuts():
-    hotkeys_html = resources.doc(N_('hotkeys.html'))
+    hotkeys_html = resources.data_path(N_('hotkeys.html'))
+    if utils.is_win32():
+        hotkeys_url = 'file:///' + hotkeys_html.replace('\\', '/')
+    else:
+        hotkeys_url = 'file://' + hotkeys_html
+    if not os.path.isfile(hotkeys_html):
+        hotkeys_url = 'https://git-cola.gitlab.io/share/doc/git-cola/hotkeys.html'
     try:
         from qtpy import QtWebEngineWidgets
     except (ImportError, qtpy.PythonQtError):
         # Redhat disabled QtWebKit in their Qt build but don't punish the users
-        webbrowser.open_new_tab('file://' + hotkeys_html)
+        webbrowser.open_new_tab(hotkeys_url)
         return
 
-    html = core.read(hotkeys_html)
-
     parent = qtutils.active_window()
     widget = QtWidgets.QDialog(parent)
     widget.setWindowModality(Qt.WindowModal)
     widget.setWindowTitle(N_('Shortcuts'))
 
     web = QtWebEngineWidgets.QWebEngineView()
-    web.setHtml(html)
+    web.setUrl(QtCore.QUrl(hotkeys_url))
 
     layout = qtutils.hbox(defs.no_margin, defs.spacing, web)
     widget.setLayout(layout)
