@@ -83,7 +83,7 @@ class FormWidget(QtWidgets.QWidget):
 
     def _item_config_changed(self, config, widget):
         def runner():
-            value = widget.current_data()
+            value = widget.value()
             cmds.do(prefs.SetConfig, self.model, self.source, config, value)
 
         return runner
@@ -162,7 +162,17 @@ scissors
         self.autotemplate = qtutils.checkbox(checked=False)
         self.merge_diffstat = qtutils.checkbox(checked=True)
         self.display_untracked = qtutils.checkbox(checked=True)
+        self.rebase_update_refs = qtutils.checkbox(checked=False)
         self.show_path = qtutils.checkbox(checked=True)
+        self.update_index = qtutils.checkbox(checked=True)
+        self.http_proxy = QtWidgets.QLineEdit()
+
+        tooltip = N_(
+            'Enable file system change monitoring using '
+            'inotify on Linux and win32event on Windows'
+        )
+        self.inotify = qtutils.checkbox(checked=True)
+        self.inotify.setToolTip(tooltip)
 
         self.logdate = qtutils.combo(prefs.date_formats())
         tooltip = N_(
@@ -171,42 +181,45 @@ scissors
         )
         self.logdate.setToolTip(tooltip)
 
-        tooltip = N_('Detect conflict markers in unmerged files')
-        self.check_conflicts = qtutils.checkbox(checked=True, tooltip=tooltip)
-
         tooltip = N_('Use gravatar.com to lookup icons for author emails')
         self.enable_gravatar = qtutils.checkbox(checked=True, tooltip=tooltip)
 
-        tooltip = N_('Prevent "Stage" from staging all files when nothing is selected')
-        self.safe_mode = qtutils.checkbox(checked=False, tooltip=tooltip)
-
         tooltip = N_('Enable path autocompletion in tools')
         self.autocomplete_paths = qtutils.checkbox(checked=True, tooltip=tooltip)
 
-        tooltip = N_('Check whether a commit has been published when amending')
-        self.check_published_commits = qtutils.checkbox(checked=True, tooltip=tooltip)
+        self.proxy_spacer = QtWidgets.QLabel()
+        tooltip = N_('Enable detection and configuration of HTTP proxy settings')
+        self.autodetect_proxy = qtutils.checkbox(checked=True, tooltip=tooltip)
 
-        self.add_row(N_('User Name'), self.name)
-        self.add_row(N_('Email Address'), self.email)
+        self.add_row(N_('Name'), self.name)
+        self.add_row(N_('Email'), self.email)
+        self.add_row(N_('Patches Directory'), self.patches_directory)
         self.add_row(N_('Log Date Format'), self.logdate)
         self.add_row(N_('Commit Message Cleanup'), self.commit_cleanup)
         self.add_row(N_('Merge Verbosity'), self.merge_verbosity)
         self.add_row(N_('Number of Diff Context Lines'), self.diff_context)
-        self.add_row(N_('Patches Directory'), self.patches_directory)
         self.add_row(N_('Summarize Merge Commits'), self.merge_summary)
+        self.add_row(N_('Enable Filesystem Monitoring'), self.inotify)
+        self.add_row(
+            N_('Update stacked branches/refs when rebasing'),
+            self.rebase_update_refs,
+        )
         self.add_row(
             N_('Automatically Load Commit Message Template'), self.autotemplate
         )
         self.add_row(N_('Show Full Paths in the Window Title'), self.show_path)
         self.add_row(N_('Show Diffstat After Merge'), self.merge_diffstat)
         self.add_row(N_('Display Untracked Files'), self.display_untracked)
-        self.add_row(N_('Detect Conflict Markers'), self.check_conflicts)
         self.add_row(N_('Enable Gravatar Icons'), self.enable_gravatar)
-        self.add_row(N_('Safe Mode'), self.safe_mode)
         self.add_row(N_('Autocomplete Paths'), self.autocomplete_paths)
+        self.add_row(N_('Update Index on Startup'), self.update_index)
+
+        self.add_row('', self.proxy_spacer)
         self.add_row(
-            N_('Check Published Commits when Amending'), self.check_published_commits
+            N_('Automatically Detect and Configure Proxy Settings'),
+            self.autodetect_proxy,
         )
+        self.add_row(N_('HTTP Proxy URL'), self.http_proxy)
 
         self.set_config({
             prefs.AUTOTEMPLATE: (self.autotemplate, Defaults.autotemplate),
@@ -214,10 +227,9 @@ scissors
                 self.autocomplete_paths,
                 Defaults.autocomplete_paths,
             ),
-            prefs.CHECK_CONFLICTS: (self.check_conflicts, Defaults.check_conflicts),
-            prefs.CHECK_PUBLISHED_COMMITS: (
-                self.check_published_commits,
-                Defaults.check_published_commits,
+            prefs.AUTODETECT_PROXY: (
+                self.autodetect_proxy,
+                Defaults.autodetect_proxy,
             ),
             prefs.COMMIT_CLEANUP: (self.commit_cleanup, Defaults.commit_cleanup),
             prefs.DIFFCONTEXT: (self.diff_context, Defaults.diff_context),
@@ -226,6 +238,8 @@ scissors
                 Defaults.display_untracked,
             ),
             prefs.ENABLE_GRAVATAR: (self.enable_gravatar, Defaults.enable_gravatar),
+            prefs.HTTP_PROXY: (self.http_proxy, Defaults.http_proxy),
+            prefs.INOTIFY: (self.inotify, Defaults.inotify),
             prefs.LOGDATE: (self.logdate, Defaults.logdate),
             prefs.MERGE_DIFFSTAT: (self.merge_diffstat, Defaults.merge_diffstat),
             prefs.MERGE_SUMMARY: (self.merge_summary, Defaults.merge_summary),
@@ -234,25 +248,30 @@ scissors
                 self.patches_directory,
                 Defaults.patches_directory,
             ),
-            prefs.SAFE_MODE: (self.safe_mode, Defaults.safe_mode),
+            prefs.REBASE_UPDATE_REFS: (
+                self.rebase_update_refs,
+                Defaults.rebase_update_refs,
+            ),
             prefs.SHOW_PATH: (self.show_path, Defaults.show_path),
             prefs.USER_NAME: (self.name, ''),
             prefs.USER_EMAIL: (self.email, ''),
+            prefs.UPDATE_INDEX: (self.update_index, Defaults.update_index),
         })
 
 
 class SettingsFormWidget(FormWidget):
     def __init__(self, context, model, parent):
         FormWidget.__init__(self, context, model, parent)
-
-        self.fixed_font = QtWidgets.QFontComboBox()
-        self.font_size = standard.SpinBox(value=12, mini=6, maxi=192)
-
         self.maxrecent = standard.SpinBox(maxi=99)
         self.tabwidth = standard.SpinBox(maxi=42)
         self.textwidth = standard.SpinBox(maxi=150)
 
         self.editor = QtWidgets.QLineEdit()
+        self.editor.setToolTip(N_('The main GUI editor that must block until it exits'))
+        self.background_editor = QtWidgets.QLineEdit()
+        self.background_editor.setToolTip(
+            N_('A non-blocking GUI editor that is launched in the background')
+        )
         self.historybrowser = QtWidgets.QLineEdit()
         self.blameviewer = QtWidgets.QLineEdit()
         self.difftool = QtWidgets.QLineEdit()
@@ -264,27 +283,45 @@ class SettingsFormWidget(FormWidget):
         self.sort_bookmarks = qtutils.checkbox()
         self.save_window_settings = qtutils.checkbox()
         self.check_spelling = qtutils.checkbox()
-        self.expandtab = qtutils.checkbox()
+        tooltip = N_('Detect conflict markers in unmerged files')
+        self.check_conflicts = qtutils.checkbox(checked=True, tooltip=tooltip)
+        self.expandtab = qtutils.checkbox(tooltip=N_('Insert tabs instead of spaces'))
+        tooltip = N_('Prevent "Stage" from staging all files when nothing is selected')
+        self.safe_mode = qtutils.checkbox(checked=False, tooltip=tooltip)
+        tooltip = N_('Check whether a commit has been published when amending')
+        self.check_published_commits = qtutils.checkbox(checked=True, tooltip=tooltip)
+        tooltip = N_(
+            'Refresh repository state whenever the window is focused or un-minimized'
+        )
+        self.refresh_on_focus = qtutils.checkbox(checked=False, tooltip=tooltip)
         self.resize_browser_columns = qtutils.checkbox(checked=False)
+        tooltip = N_('Emit notifications when commits are pushed.')
+        self.notifyonpush = qtutils.checkbox(checked=False, tooltip=tooltip)
 
-        self.add_row(N_('Fixed-Width Font'), self.fixed_font)
-        self.add_row(N_('Font Size'), self.font_size)
         self.add_row(N_('Text Width'), self.textwidth)
         self.add_row(N_('Tab Width'), self.tabwidth)
         self.add_row(N_('Editor'), self.editor)
+        self.add_row(N_('Background Editor'), self.background_editor)
         self.add_row(N_('History Browser'), self.historybrowser)
         self.add_row(N_('Blame Viewer'), self.blameviewer)
         self.add_row(N_('Diff Tool'), self.difftool)
         self.add_row(N_('Merge Tool'), self.mergetool)
-        self.add_row(N_('Recent repository count'), self.maxrecent)
+        self.add_row(N_('Recent Repository Count'), self.maxrecent)
         self.add_row(N_('Auto-Wrap Lines'), self.linebreak)
-        self.add_row(N_('Insert spaces instead of tabs'), self.expandtab)
-        self.add_row(N_('Sort bookmarks alphabetically'), self.sort_bookmarks)
+        self.add_row(N_('Insert Spaces Instead of Tabs'), self.expandtab)
+        self.add_row(
+            N_('Check Published Commits when Amending'), self.check_published_commits
+        )
+        self.add_row(N_('Sort Bookmarks Alphabetically'), self.sort_bookmarks)
+        self.add_row(N_('Safe Mode'), self.safe_mode)
+        self.add_row(N_('Detect Conflict Markers'), self.check_conflicts)
         self.add_row(N_('Keep *.orig Merge Backups'), self.keep_merge_backups)
         self.add_row(N_('Save GUI Settings'), self.save_window_settings)
+        self.add_row(N_('Refresh on Focus'), self.refresh_on_focus)
         self.add_row(N_('Resize File Browser columns'), self.resize_browser_columns)
-        self.add_row(N_('Check spelling'), self.check_spelling)
-        self.add_row(N_('Ctrl+MouseWheel to Zoom'), self.mouse_zoom)
+        self.add_row(N_('Check Spelling'), self.check_spelling)
+        self.add_row(N_('Ctrl + MouseWheel to Zoom'), self.mouse_zoom)
+        self.add_row(N_('Notify on Push'), self.notifyonpush)
 
         self.set_config({
             prefs.SAVEWINDOWSETTINGS: (
@@ -299,52 +336,51 @@ class SettingsFormWidget(FormWidget):
             prefs.SORT_BOOKMARKS: (self.sort_bookmarks, Defaults.sort_bookmarks),
             prefs.DIFFTOOL: (self.difftool, Defaults.difftool),
             prefs.EDITOR: (self.editor, fallback_editor()),
+            prefs.BACKGROUND_EDITOR: (
+                self.background_editor,
+                Defaults.background_editor,
+            ),
             prefs.HISTORY_BROWSER: (
                 self.historybrowser,
                 prefs.default_history_browser(),
             ),
             prefs.BLAME_VIEWER: (self.blameviewer, Defaults.blame_viewer),
+            prefs.CHECK_CONFLICTS: (self.check_conflicts, Defaults.check_conflicts),
+            prefs.CHECK_PUBLISHED_COMMITS: (
+                self.check_published_commits,
+                Defaults.check_published_commits,
+            ),
             prefs.MERGE_KEEPBACKUP: (
                 self.keep_merge_backups,
                 Defaults.merge_keep_backup,
             ),
             prefs.MERGETOOL: (self.mergetool, Defaults.mergetool),
+            prefs.REFRESH_ON_FOCUS: (self.refresh_on_focus, Defaults.refresh_on_focus),
             prefs.RESIZE_BROWSER_COLUMNS: (
                 self.resize_browser_columns,
                 Defaults.resize_browser_columns,
             ),
+            prefs.SAFE_MODE: (self.safe_mode, Defaults.safe_mode),
             prefs.SPELL_CHECK: (self.check_spelling, Defaults.spellcheck),
             prefs.MOUSE_ZOOM: (self.mouse_zoom, Defaults.mouse_zoom),
+            prefs.NOTIFY_ON_PUSH: (self.notifyonpush, Defaults.notifyonpush),
         })
 
-        self.fixed_font.currentFontChanged.connect(self.current_font_changed)
-        self.font_size.valueChanged.connect(self.font_size_changed)
-
-    def update_from_config(self):
-        """Update widgets to the current config values"""
-        FormWidget.update_from_config(self)
-        context = self.context
-
-        with qtutils.BlockSignals(self.fixed_font):
-            font = qtutils.diff_font(context)
-            self.fixed_font.setCurrentFont(font)
-
-        with qtutils.BlockSignals(self.font_size):
-            font_size = font.pointSize()
-            self.font_size.setValue(font_size)
-
-    def font_size_changed(self, size):
-        font = self.fixed_font.currentFont()
-        font.setPointSize(size)
-        cmds.do(prefs.SetConfig, self.model, 'global', prefs.FONTDIFF, font.toString())
-
-    def current_font_changed(self, font):
-        cmds.do(prefs.SetConfig, self.model, 'global', prefs.FONTDIFF, font.toString())
-
 
 class AppearanceFormWidget(FormWidget):
     def __init__(self, context, model, parent):
         FormWidget.__init__(self, context, model, parent)
+        # Fonts
+        font = self.font()
+        font_size = context.cfg.get(prefs.FONTSIZE, font.pointSize())
+        self.default_font_size = font_size
+        self.fixed_font = QtWidgets.QFontComboBox()
+        self.fixed_font_size = standard.SpinBox(value=12, mini=6, maxi=192)
+        self.fixed_font_size.setToolTip(
+            N_('The font size for the fixed-width diff font')
+        )
+        self.font_size = standard.SpinBox(value=font_size, mini=6, maxi=192)
+        self.font_size.setToolTip(N_('The font size for the main UI elements'))
         # Theme selectors
         self.themes = themes.get_all_themes()
         self.theme = qtutils.combo_mapped(themes.options(themes=self.themes))
@@ -360,16 +396,23 @@ class AppearanceFormWidget(FormWidget):
         self.status_indent = qtutils.checkbox()
         self.block_cursor = qtutils.checkbox(checked=True)
 
+        self.add_row(N_('Fixed-Width Font'), self.fixed_font)
+        self.add_row(N_('Font Size'), self.fixed_font_size)
+        self.add_row(N_('Font Size (UI)'), self.font_size)
         self.add_row(N_('GUI theme'), self.theme)
         self.add_row(N_('Icon theme'), self.icon_theme)
         self.add_row(N_('High DPI'), self.high_dpi)
-        self.add_row(N_('Bold on dark headers instead of italic'), self.bold_headers)
-        self.add_row(N_('Show file counts in Status titles'), self.status_show_totals)
+        self.add_row(N_('Bold on Dark Headers Instead of Italic'), self.bold_headers)
+        self.add_row(N_('Show File Counts in Status Titles'), self.status_show_totals)
         self.add_row(N_('Indent Status paths'), self.status_indent)
-        self.add_row(N_('Use a block cursor in diff editors'), self.block_cursor)
+        self.add_row(N_('Use a Block Cursor in Diff Editors'), self.block_cursor)
 
         self.set_config({
             prefs.BOLD_HEADERS: (self.bold_headers, Defaults.bold_headers),
+            prefs.FONTSIZE: (
+                self.font_size,
+                self.default_font_size,
+            ),
             prefs.HIDPI: (self.high_dpi, Defaults.hidpi),
             prefs.STATUS_SHOW_TOTALS: (
                 self.status_show_totals,
@@ -381,6 +424,9 @@ class AppearanceFormWidget(FormWidget):
             prefs.BLOCK_CURSOR: (self.block_cursor, Defaults.block_cursor),
         })
 
+        self.fixed_font.currentFontChanged.connect(self.current_font_changed)
+        self.fixed_font_size.valueChanged.connect(self.diff_font_size_changed)
+        self.font_size.valueChanged.connect(self.font_size_changed)
         self.theme.currentIndexChanged.connect(self._theme_changed)
 
     def _theme_changed(self, theme_idx):
@@ -390,7 +436,7 @@ class AppearanceFormWidget(FormWidget):
             theme = self.themes[theme_idx]
         except IndexError:
             return
-        icon_theme = self.icon_theme.current_data()
+        icon_theme = self.icon_theme.value()
         if theme.name == 'default':
             if icon_theme in ('light', 'dark'):
                 self.icon_theme.set_value('default')
@@ -401,6 +447,32 @@ class AppearanceFormWidget(FormWidget):
             if icon_theme in ('default', 'dark'):
                 self.icon_theme.set_value('light')
 
+    def update_from_config(self):
+        """Update widgets to the current config values"""
+        FormWidget.update_from_config(self)
+        context = self.context
+
+        with qtutils.BlockSignals(self.fixed_font):
+            font = qtutils.diff_font(context)
+            self.fixed_font.setCurrentFont(font)
+
+        with qtutils.BlockSignals(self.fixed_font_size):
+            font_size = font.pointSize()
+            self.fixed_font_size.setValue(font_size)
+
+    def diff_font_size_changed(self, size):
+        """The diff font size was changed"""
+        font = self.fixed_font.currentFont()
+        font.setPointSize(size)
+        cmds.do(prefs.SetConfig, self.model, 'global', prefs.FONTDIFF, font.toString())
+
+    def font_size_changed(self, size):
+        """The UI font size was changed"""
+        cmds.do(prefs.SetConfig, self.model, 'global', prefs.FONTSIZE, size)
+
+    def current_font_changed(self, font):
+        cmds.do(prefs.SetConfig, self.model, 'global', prefs.FONTDIFF, font.toString())
+
 
 class AppearanceWidget(QtWidgets.QWidget):
     def __init__(self, form, parent):
@@ -437,8 +509,8 @@ class PreferencesView(standard.Dialog):
 
         self.tab_bar = QtWidgets.QTabBar()
         self.tab_bar.setDrawBase(False)
-        self.tab_bar.addTab(N_('All Repositories'))
         self.tab_bar.addTab(N_('Current Repository'))
+        self.tab_bar.addTab(N_('All Repositories'))
         self.tab_bar.addTab(N_('Settings'))
         self.tab_bar.addTab(N_('Appearance'))
 
@@ -449,8 +521,8 @@ class PreferencesView(standard.Dialog):
         self.appearance = AppearanceWidget(self.appearance_form, self)
 
         self.stack_widget = QtWidgets.QStackedWidget()
-        self.stack_widget.addWidget(self.user_form)
         self.stack_widget.addWidget(self.repo_form)
+        self.stack_widget.addWidget(self.user_form)
         self.stack_widget.addWidget(self.options_form)
         self.stack_widget.addWidget(self.appearance)
 
@@ -459,6 +531,14 @@ class PreferencesView(standard.Dialog):
         self.button_layout = qtutils.hbox(
             defs.no_margin, defs.spacing, qtutils.STRETCH, self.close_button
         )
+        # If the user already has the user.email and user.name configured then default
+        # to editing the current repo's config instead of the user-wide settings.
+        if self.context.cfg.get('user.name') and self.context.cfg.get('user.email'):
+            index = 0
+        else:
+            index = 1
+        self.stack_widget.setCurrentIndex(index)
+        self.tab_bar.setCurrentIndex(index)
 
         self.main_layout = qtutils.vbox(
             defs.margin,
@@ -475,7 +555,7 @@ class PreferencesView(standard.Dialog):
         qtutils.connect_button(self.close_button, self.accept)
         qtutils.add_close_action(self)
 
-        self.update_widget(0)
+        self.update_widget(index)
 
     def update_widget(self, idx):
         widget = self.stack_widget.widget(idx)
