@@ -62,17 +62,21 @@ class WidgetMixin:
         return self.__class__.__name__.lower()
 
     def save_state(self, settings=None):
+        """Save tool settings to the ~/.config/git-cola/settings file"""
         save = True
+        sync = True
         context = getattr(self, 'context', None)
         if context:
             cfg = context.cfg
             save = cfg.get('cola.savewindowsettings', default=True)
+            sync = cfg.get('cola.sync', default=True)
         if save:
             if settings is None:
                 settings = Settings.read()
-            settings.save_gui_state(self)
+            settings.save_gui_state(self, sync=sync)
 
     def restore_state(self, settings=None):
+        """Read and apply saved tool settings"""
         if settings is None:
             settings = Settings.read()
         state = settings.get_gui_state(self)
@@ -83,8 +87,7 @@ class WidgetMixin:
         return result
 
     def apply_state(self, state):
-        """Imports data for view save/restore"""
-
+        """Import data for view save/restore"""
         width = utils.asint(state.get('width'))
         height = utils.asint(state.get('height'))
         x = utils.asint(state.get('x'))
@@ -117,19 +120,29 @@ class WidgetMixin:
         return state
 
     def save_settings(self, settings=None):
+        """Save tool state using the specified settings backend"""
         return self.save_state(settings=settings)
 
     def closeEvent(self, event):
+        """Save settings when the top-level widget is closed"""
         self.save_settings()
         self.closed.emit(self)
         self.Base.closeEvent(self, event)
 
     def init_size(self, parent=None, settings=None, width=0, height=0):
+        """Set a tool's initial size"""
         if not width:
             width = defs.dialog_w
         if not height:
             height = defs.dialog_h
-        self.init_state(settings, self.resize_to_parent, parent, width, height)
+        self.init_state(
+            settings,
+            self.resize_to_parent,
+            parent,
+            width,
+            height,
+            use_parent_height=True,
+        )
 
     def init_state(self, settings, callback, *args, **kwargs):
         """Restore saved settings or set the initial location"""
@@ -137,9 +150,11 @@ class WidgetMixin:
             callback(*args, **kwargs)
             self.center()
 
-    def resize_to_parent(self, parent, w, h):
+    def resize_to_parent(self, parent, w, h, use_parent_height=True):
         """Set the initial size of the widget"""
-        width, height = qtutils.default_size(parent, w, h)
+        width, height = qtutils.default_size(
+            parent, w, h, use_parent_height=use_parent_height
+        )
         self.resize(width, height)
 
 
@@ -158,6 +173,14 @@ class MainWindowMixin(WidgetMixin):
         self.default_state = self.saveState(self.widget_version)
         super().init_state(settings, callback, *args, **kwargs)
 
+    def layout_state(self):
+        """Return the Qt layout state"""
+        return self.saveState(self.widget_version)
+
+    def apply_layout(self, value):
+        """Apply binary Qt layout state"""
+        self.restoreState(value, self.widget_version)
+
     def export_state(self):
         """Exports data for save/restore"""
         state = WidgetMixin.export_state(self)
@@ -174,7 +197,12 @@ class MainWindowMixin(WidgetMixin):
             else:
                 settings = context.settings
                 settings.load()
-            settings.add_recent(core.getcwd(), prefs.maxrecent(context))
+            try:
+                cwd = core.getcwd()
+            except FileNotFoundError:
+                pass
+            else:
+                settings.add_recent(cwd, prefs.maxrecent(context))
         return WidgetMixin.save_settings(self, settings=settings)
 
     def apply_state(self, state):
@@ -193,7 +221,7 @@ class MainWindowMixin(WidgetMixin):
 
         self.lock_layout = state.get('lock_layout', self.lock_layout)
         self.update_dockwidget_lock_state()
-        self.update_dockwidget_tooltips()
+        self.update_dockwidget_floating_state()
 
         return result
 
@@ -214,12 +242,13 @@ class MainWindowMixin(WidgetMixin):
                 | QDockWidget.DockWidgetMovable
             )
         for widget in self.dockwidgets:
-            widget.titleBarWidget().update_tooltips()
             widget.setFeatures(features)
+            widget.titleBarWidget().update_floating()
 
-    def update_dockwidget_tooltips(self):
+    def update_dockwidget_floating_state(self):
+        """Update the floating state for all dock widgets"""
         for widget in self.dockwidgets:
-            widget.titleBarWidget().update_tooltips()
+            widget.titleBarWidget().update_floating()
 
 
 class ListWidget(QtWidgets.QListWidget):
@@ -790,21 +819,39 @@ class ProgressTickThread(QtCore.QThread):
 
 class SpinBox(QtWidgets.QSpinBox):
     def __init__(
-        self, parent=None, value=None, mini=1, maxi=99999, step=0, prefix='', suffix=''
+        self,
+        parent=None,
+        value=None,
+        mini=1,
+        maxi=99999,
+        step=0,
+        prefix='',
+        suffix='',
+        tooltip='',
+        wrap=False,
     ):
         QtWidgets.QSpinBox.__init__(self, parent)
+        self.setAlignment(Qt.AlignRight)
+        self.setButtonSymbols(QtWidgets.QAbstractSpinBox.PlusMinus)
         self.setPrefix(prefix)
         self.setSuffix(suffix)
         self.setWrapping(True)
         self.setMinimum(mini)
         self.setMaximum(maxi)
+        self.setWrapping(wrap)
         if step:
             self.setSingleStep(step)
         if value is not None:
-            self.setValue(value)
+            self.set_value(value)
         text_width = qtutils.text_width(self.font(), 'MMMMMM')
         width = max(self.minimumWidth(), text_width)
         self.setMinimumWidth(width)
+        if tooltip:
+            self.setToolTip(tooltip)
+
+    def set_value(self, value):
+        """Set the spinbox value directly"""
+        self.setValue(value)
 
 
 class DirectoryPathLineEdit(QtWidgets.QWidget):
