@@ -19,6 +19,7 @@ from . import defs
 
 
 def get_stripped(widget):
+    """Return a text value without any leading or trailing whitespace"""
     return widget.get().strip()
 
 
@@ -26,15 +27,11 @@ class LineEdit(QtWidgets.QLineEdit):
     cursor_changed = Signal(int, int)
     esc_pressed = Signal()
 
-    def __init__(self, parent=None, row=1, get_value=None, clear_button=False):
+    def __init__(self, parent=None, row=1, clear_button=False):
         QtWidgets.QLineEdit.__init__(self, parent)
         self._row = row
-        if get_value is None:
-            get_value = get_stripped
-        self._get_value = get_value
         self.cursor_position = LineEditCursorPosition(self, row)
         self.menu_actions = []
-
         if clear_button and hasattr(self, 'setClearButtonEnabled'):
             self.setClearButtonEnabled(True)
 
@@ -44,7 +41,7 @@ class LineEdit(QtWidgets.QLineEdit):
 
     def value(self):
         """Return the processed value, e.g. stripped"""
-        return self._get_value(self)
+        return get_stripped(self)
 
     def set_value(self, value, block=False):
         """Update the widget to the specified value"""
@@ -56,7 +53,7 @@ class LineEdit(QtWidgets.QLineEdit):
 
     def _set_value(self, value):
         """Implementation helper to update the widget to the specified value"""
-        pos = self.cursorPosition()
+        pos = utils.clamp_zero(self.cursorPosition(), len(value))
         self.setText(value)
         self.setCursorPosition(pos)
 
@@ -87,13 +84,10 @@ class LineEditCursorPosition:
 
 
 class BaseTextEditExtension(QtCore.QObject):
-    def __init__(self, widget, get_value, readonly):
+    def __init__(self, widget, readonly):
         QtCore.QObject.__init__(self, widget)
         self.widget = widget
         self.cursor_position = TextEditCursorPosition(widget, self)
-        if get_value is None:
-            get_value = get_stripped
-        self._get_value = get_value
         self._tabwidth = 8
         self._readonly = readonly
         self._init_flags()
@@ -102,8 +96,10 @@ class BaseTextEditExtension(QtCore.QObject):
     def _init_flags(self):
         widget = self.widget
         widget.setMinimumSize(QtCore.QSize(10, 10))
-        widget.setWordWrapMode(QtGui.QTextOption.WordWrap)
-        widget.setLineWrapMode(widget.NoWrap)
+        widget.setWordWrapMode(
+            getattr(widget, 'word_wrap_mode', QtGui.QTextOption.WordWrap)
+        )
+        widget.setLineWrapMode(widget.__class__.NoWrap)
         if self._readonly:
             widget.setReadOnly(True)
             widget.setAcceptDrops(False)
@@ -119,7 +115,7 @@ class BaseTextEditExtension(QtCore.QObject):
 
     def value(self):
         """Return a safe value, e.g. a stripped value"""
-        return self._get_value(self.widget)
+        return get_stripped(self.widget)
 
     def set_value(self, value, block=False):
         """Update the widget to the specified value"""
@@ -155,13 +151,10 @@ class BaseTextEditExtension(QtCore.QObject):
             self.widget.setTextCursor(cursor)
         else:
             # If none of the above applied then restore the cursor position.
-            position = max(0, min(offset, len(value) - 1))
+            position = utils.clamp_zero(offset, len(value))
             cursor = self.widget.textCursor()
             cursor.setPosition(position)
             self.widget.setTextCursor(cursor)
-            cursor = self.widget.textCursor()
-            cursor.movePosition(QtGui.QTextCursor.StartOfLine)
-            self.widget.setTextCursor(cursor)
 
     def set_cursor_position(self, new_position):
         cursor = self.widget.textCursor()
@@ -272,7 +265,9 @@ class BaseTextEditExtension(QtCore.QObject):
 class PlainTextEditExtension(BaseTextEditExtension):
     def set_linebreak(self, brk):
         if brk:
-            wrapmode = QtWidgets.QPlainTextEdit.WidgetWidth
+            wrapmode = getattr(
+                self.widget, 'line_wrap_mode', QtWidgets.QPlainTextEdit.WidgetWidth
+            )
         else:
             wrapmode = QtWidgets.QPlainTextEdit.NoWrap
         self.widget.setLineWrapMode(wrapmode)
@@ -281,14 +276,28 @@ class PlainTextEditExtension(BaseTextEditExtension):
 class PlainTextEdit(QtWidgets.QPlainTextEdit):
     cursor_changed = Signal(int, int)
     leave = Signal()
+    mouse_zoomed = Signal()
 
-    def __init__(self, parent=None, get_value=None, readonly=False, options=None):
+    def __init__(
+        self,
+        parent=None,
+        readonly=False,
+        options=None,
+        line_wrap_mode=None,
+        word_wrap_mode=None,
+    ):
         QtWidgets.QPlainTextEdit.__init__(self, parent)
-        self.ext = PlainTextEditExtension(self, get_value, readonly)
-        self.cursor_position = self.ext.cursor_position
+        if line_wrap_mode is None:
+            line_wrap_mode = QtWidgets.QPlainTextEdit.WidgetWidth
+        if word_wrap_mode is None:
+            word_wrap_mode = QtGui.QTextOption.WordWrap
         self.mouse_zoom = True
         self.options = options
         self.menu_actions = []
+        self.line_wrap_mode = line_wrap_mode
+        self.word_wrap_mode = word_wrap_mode
+        self.ext = PlainTextEditExtension(self, readonly)
+        self.cursor_position = self.ext.cursor_position
 
     def get(self):
         """Return the raw Unicode value from Qt"""
@@ -306,6 +315,14 @@ class PlainTextEdit(QtWidgets.QPlainTextEdit):
         """Return the cursor offset and selected text"""
         return self.ext.offset_and_selection()
 
+    def selected_line_range(self):
+        """Return the start and end lines corresponding to the current selection"""
+        offset, selection = self.offset_and_selection()
+        content = self.get()
+        start_line = content[:offset].count('\n')
+        span = max(1, selection.count('\n'))
+        return start_line + 1, span
+
     def set_value(self, value, block=False):
         self.ext.set_value(value, block=block)
 
@@ -323,8 +340,8 @@ class PlainTextEdit(QtWidgets.QPlainTextEdit):
             with qtutils.BlockSignals(self.options.enable_word_wrapping):
                 self.options.enable_word_wrapping.setChecked(enabled)
         if enabled:
-            self.setWordWrapMode(QtGui.QTextOption.WordWrap)
-            self.setLineWrapMode(QtWidgets.QPlainTextEdit.WidgetWidth)
+            self.setWordWrapMode(self.word_wrap_mode)
+            self.setLineWrapMode(self.line_wrap_mode)
         else:
             self.setWordWrapMode(QtGui.QTextOption.NoWrap)
             self.setLineWrapMode(QtWidgets.QPlainTextEdit.NoWrap)
@@ -358,6 +375,7 @@ class PlainTextEdit(QtWidgets.QPlainTextEdit):
             event.ignore()
             return
         super().wheelEvent(event)
+        self.mouse_zoomed.emit()
 
     def create_context_menu(self, event_pos):
         """Create a custom context menu"""
@@ -499,8 +517,7 @@ def text_matches(case_sensitive, a, b):
 
 class TextEditExtension(BaseTextEditExtension):
     def init(self):
-        widget = self.widget
-        widget.setAcceptRichText(False)
+        self.widget.setAcceptRichText(False)
 
     def set_linebreak(self, brk):
         if brk:
@@ -517,12 +534,12 @@ class TextEdit(QtWidgets.QTextEdit):
     cursor_changed = Signal(int, int)
     leave = Signal()
 
-    def __init__(self, parent=None, get_value=None, readonly=False):
+    def __init__(self, parent=None, readonly=False):
         QtWidgets.QTextEdit.__init__(self, parent)
-        self.ext = TextEditExtension(self, get_value, readonly)
-        self.cursor_position = self.ext.cursor_position
         self.expandtab_enabled = False
         self.menu_actions = []
+        self.ext = TextEditExtension(self, readonly)
+        self.cursor_position = self.ext.cursor_position
 
     def get(self):
         """Return the raw Unicode value from Qt"""
@@ -637,62 +654,26 @@ class MonoTextEdit(PlainTextEdit):
         self.setFont(qtutils.diff_font(context))
 
 
-def get_value_hinted(widget):
-    text = get_stripped(widget)
-    hint = get(widget.hint)
-    if text == hint:
-        return ''
-    return text
-
-
 class HintWidget(QtCore.QObject):
-    """Extend a widget to provide hint messages
-
-    This primarily exists because setPlaceholderText() is only available
-    in Qt5, so this class provides consistent behavior across versions.
-
-    """
+    """Set placeholder text and apply palettes to convey errors"""
 
     def __init__(self, widget, hint):
         QtCore.QObject.__init__(self, widget)
         self._widget = widget
         self._hint = hint
         self._is_error = False
-
-        self.modern = modern = hasattr(widget, 'setPlaceholderText')
-        if modern:
-            widget.setPlaceholderText(hint)
-
-        # Palette for normal text
-        QPalette = QtGui.QPalette
-        palette = widget.palette()
-
-        hint_color = palette.color(QPalette.Disabled, QPalette.Text)
+        self._error_seen = False
+        self._default_style = ''
+        widget.setPlaceholderText(hint)
         error_bg_color = QtGui.QColor(Qt.red).darker()
         error_fg_color = QtGui.QColor(Qt.white)
-
-        hint_rgb = qtutils.rgb_css(hint_color)
         error_bg_rgb = qtutils.rgb_css(error_bg_color)
         error_fg_rgb = qtutils.rgb_css(error_fg_color)
-
         env = {
             'name': widget.__class__.__name__,
             'error_fg_rgb': error_fg_rgb,
             'error_bg_rgb': error_bg_rgb,
-            'hint_rgb': hint_rgb,
         }
-
-        self._default_style = ''
-
-        self._hint_style = (
-            """
-            %(name)s {
-                color: %(hint_rgb)s;
-            }
-        """
-            % env
-        )
-
         self._error_style = (
             """
             %(name)s {
@@ -703,22 +684,10 @@ class HintWidget(QtCore.QObject):
             % env
         )
 
-    def init(self):
-        """Deferred initialization"""
-        if self.modern:
-            self.widget().setPlaceholderText(self.value())
-        else:
-            self.widget().installEventFilter(self)
-            self.enable(True)
-
     def widget(self):
         """Return the parent text widget"""
         return self._widget
 
-    def active(self):
-        """Return True when hint-mode is active"""
-        return self.value() == get_stripped(self._widget)
-
     def value(self):
         """Return the current hint text"""
         return self._hint
@@ -726,120 +695,66 @@ class HintWidget(QtCore.QObject):
     def set_error(self, is_error):
         """Enable/disable error mode"""
         self._is_error = is_error
-        self.refresh()
+        self.refresh_palette()
 
     def set_value(self, hint):
         """Change the hint text"""
-        if self.modern:
-            self._hint = hint
-            self._widget.setPlaceholderText(hint)
-        else:
-            # If hint-mode is currently active, re-activate it
-            active = self.active()
-            self._hint = hint
-            if active or self.active():
-                self.enable(True)
+        self._hint = hint
+        self._widget.setPlaceholderText(hint)
 
-    def enable(self, enable):
-        """Enable/disable hint-mode"""
-        if not self.modern:
-            if enable and self._hint:
-                self._widget.set_value(self._hint, block=True)
-                self._widget.cursor_position.reset()
-            else:
-                self._widget.clear()
-        self._update_palette(enable)
-
-    def refresh(self):
-        """Update the palette to match the current mode"""
-        self._update_palette(self.active())
-
-    def _update_palette(self, hint):
-        """Update to palette for normal/error/hint mode"""
+    def refresh_palette(self):
+        """Update to palette for normal/error mode"""
         if self._is_error:
-            style = self._error_style
-        elif not self.modern and hint:
-            style = self._hint_style
+            if self._error_seen:
+                style = None
+            else:
+                style = self._error_style
+                self._error_seen = True
         else:
-            style = self._default_style
-        QtCore.QTimer.singleShot(
-            0, lambda: utils.catch_runtime_error(self._widget.setStyleSheet, style)
-        )
-
-    def eventFilter(self, _obj, event):
-        """Enable/disable hint-mode when focus changes"""
-        etype = event.type()
-        if etype == QtCore.QEvent.FocusIn:
-            self.focus_in()
-        elif etype == QtCore.QEvent.FocusOut:
-            self.focus_out()
-        return False
-
-    def focus_in(self):
-        """Disable hint-mode when focused"""
-        widget = self.widget()
-        if self.active():
-            self.enable(False)
-        widget.cursor_position.emit()
-
-    def focus_out(self):
-        """Re-enable hint-mode when losing focus"""
-        widget = self.widget()
-        valid, value = utils.catch_runtime_error(get, widget)
-        if not valid:
-            # The widget may have just been destroyed during application shutdown.
-            # We're receiving a focusOut event but the widget can no longer be used.
-            # This can be safely ignored.
-            return
-        if not value:
-            self.enable(True)
+            if self._error_seen:
+                self._error_seen = False
+                style = self._default_style
+            else:
+                style = None
+        if style is not None:
+            utils.catch_runtime_error(self._widget.setStyleSheet, style)
 
 
 class HintedPlainTextEdit(PlainTextEdit):
     """A hinted plain text edit"""
 
-    def __init__(self, context, hint, parent=None, readonly=False):
-        PlainTextEdit.__init__(
-            self, parent=parent, get_value=get_value_hinted, readonly=readonly
+    def __init__(
+        self,
+        context,
+        hint,
+        parent=None,
+        readonly=False,
+        line_wrap_mode=None,
+        word_wrap_mode=None,
+    ):
+        super().__init__(
+            readonly=readonly,
+            line_wrap_mode=line_wrap_mode,
+            word_wrap_mode=word_wrap_mode,
+            parent=parent,
         )
         self.hint = HintWidget(self, hint)
-        self.hint.init()
         self.context = context
         self.setFont(qtutils.diff_font(context))
         self.set_tabwidth(prefs.tabwidth(context))
-        # Refresh palettes when text changes
-        self.textChanged.connect(self.hint.refresh)
         self.set_mouse_zoom(context.cfg.get(prefs.MOUSE_ZOOM, default=True))
 
-    def set_value(self, value, block=False):
-        """Set the widget text or enable hint mode when empty"""
-        if value or self.hint.modern:
-            PlainTextEdit.set_value(self, value, block=block)
-        else:
-            self.hint.enable(True)
-
 
 class HintedTextEdit(TextEdit):
     """A hinted text edit"""
 
     def __init__(self, context, hint, parent=None, readonly=False):
-        TextEdit.__init__(
-            self, parent=parent, get_value=get_value_hinted, readonly=readonly
-        )
+        TextEdit.__init__(self, parent=parent, readonly=readonly)
         self.context = context
         self.hint = HintWidget(self, hint)
-        self.hint.init()
         # Refresh palettes when text changes
-        self.textChanged.connect(self.hint.refresh)
         self.setFont(qtutils.diff_font(context))
 
-    def set_value(self, value, block=False):
-        """Set the widget text or enable hint mode when empty"""
-        if value or self.hint.modern:
-            TextEdit.set_value(self, value, block=block)
-        else:
-            self.hint.enable(True)
-
 
 def anchor_mode(select):
     """Return the QTextCursor mode to keep/discard the cursor selection"""
@@ -850,10 +765,20 @@ def anchor_mode(select):
     return mode
 
 
-# The vim-like read-only text view
+def event_anchor_mode(event):
+    """Return the QTextCursor mode to keep/discard the selection based on the event"""
+    select = is_shift_pressed(event)
+    return anchor_mode(select)
+
+
+def is_shift_pressed(event):
+    """Return true if the Shift modifier is currently held"""
+    return event.modifiers() & Qt.ShiftModifier
 
 
 class VimMixin:
+    """Vim-like read-only text view"""
+
     def __init__(self, widget):
         self.widget = widget
         self.Base = widget.Base
@@ -984,8 +909,18 @@ class VimHintedPlainTextEdit(HintedPlainTextEdit):
     Base = HintedPlainTextEdit
     Mixin = VimMixin
 
-    def __init__(self, context, hint, parent=None):
-        HintedPlainTextEdit.__init__(self, context, hint, parent=parent, readonly=True)
+    def __init__(
+        self, context, hint, line_wrap_mode=None, word_wrap_mode=None, parent=None
+    ):
+        HintedPlainTextEdit.__init__(
+            self,
+            context,
+            hint,
+            parent=parent,
+            readonly=True,
+            line_wrap_mode=line_wrap_mode,
+            word_wrap_mode=word_wrap_mode,
+        )
         self._mixin = self.Mixin(self)
 
     def move(self, direction, select=False, n=1):
@@ -1038,12 +973,10 @@ class HintedDefaultLineEdit(LineEdit):
     """A line edit with hint text"""
 
     def __init__(self, hint, tooltip=None, parent=None):
-        LineEdit.__init__(self, parent=parent, get_value=get_value_hinted)
+        LineEdit.__init__(self, parent=parent)
         if tooltip:
             self.setToolTip(tooltip)
         self.hint = HintWidget(self, hint)
-        self.hint.init()
-        self.textChanged.connect(lambda text: self.hint.refresh())
 
 
 class HintedLineEdit(HintedDefaultLineEdit):
@@ -1150,11 +1083,10 @@ class LineNumbers(TextDecorator):
 
     def paintEvent(self, event):
         """Paint the line number"""
-        QPalette = QtGui.QPalette
         painter = QtGui.QPainter(self)
         editor = self.editor
         palette = editor.palette()
-
+        QPalette = QtGui.QPalette
         painter.fillRect(event.rect(), palette.color(QPalette.Base))
 
         content_offset = editor.contentOffset()
@@ -1192,27 +1124,105 @@ class LineNumbers(TextDecorator):
             block = block.next()
 
 
+def label_selection_timer(widget):
+    """Create a timer for handling the copy-on-click action"""
+    timer = QtCore.QTimer(widget)
+    timer.setInterval(200)
+    timer.setSingleShot(True)
+    timer.timeout.connect(lambda widget=widget: widget.setSelection(0, 0))
+    return timer
+
+
 class TextLabel(QtWidgets.QLabel):
     """A text label that elides its display"""
 
-    def __init__(self, parent=None, open_external_links=True):
+    def __init__(
+        self,
+        parent=None,
+        copy_on_click=False,
+        open_external_links=True,
+        selectable=True,
+        text_format=Qt.PlainText,
+    ):
         QtWidgets.QLabel.__init__(self, parent)
-        self._display = ''
-        self._template = ''
-        self._text = ''
+        self._copy_on_click = copy_on_click
+        self._display = ''  # The final displayed text.
+        self._template = ''  # The plaintext version of the richtext html template.
+        self._text = ''  # The text value or the rich text html.
+        self._text_format = text_format  # The QTextFormat usef for this label.
         self._elide = False
         self._metrics = QtGui.QFontMetrics(self.font())
-        policy = QtWidgets.QSizePolicy(
+
+        self.setTextFormat(text_format)
+        self.setCursor(Qt.PointingHandCursor)
+        self.setContextMenuPolicy(Qt.CustomContextMenu)
+        self.setFocusPolicy(Qt.NoFocus)
+        size_policy = QtWidgets.QSizePolicy(
             QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Minimum
         )
-        self.setSizePolicy(policy)
-        self.setTextInteractionFlags(
-            Qt.TextSelectableByMouse | Qt.LinksAccessibleByMouse
-        )
+        self.setSizePolicy(size_policy)
+        if selectable:
+            interaction_flags = Qt.TextSelectableByMouse | Qt.LinksAccessibleByMouse
+        else:
+            interaction_flags = Qt.LinksAccessibleByMouse
+        self.setTextInteractionFlags(interaction_flags)
         self.setOpenExternalLinks(open_external_links)
+        self.copy_all_action = qtutils.add_action_with_icon(
+            self,
+            icons.copy(),
+            N_('Copy All'),
+            self.copy_all,
+        )
+        self.copy_selection_action = qtutils.add_action_with_icon(
+            self,
+            icons.copy(),
+            N_('Copy Selection'),
+            self.copy_selection,
+        )
+        self.select_all_action = qtutils.add_action(
+            self, N_('Select All'), self.select_all
+        )
+        self.timer = label_selection_timer(self)
+        self.customContextMenuRequested.connect(self._context_menu)
+
+    def copy_all_callback(self):
+        """Specialized by subclasses to customize the copy-on-click behavior"""
+        self.copy_all()
+
+    def copy_all(self):
+        """Copy the text label to the clipboard"""
+        qtutils.set_clipboard(self._template)
+        self.start_selection_timer()
+
+    def copy_selection(self):
+        """Handle elided text when copying to the clipboard"""
+        text = self.selectedText()
+        if text == self._display and self._display != self._text:
+            text = self._text
+        qtutils.set_clipboard(text)
+
+    def select_all(self):
+        """Select the entire text label"""
+        if self.textFormat() == Qt.RichText:
+            self.setSelection(0, min(len(self._display), len(self._template)))
+        else:
+            self.setSelection(0, len(self._display))
 
     def elide(self):
         self._elide = True
+        return self
+
+    def align_bottom(self):
+        self.setAlignment(Qt.AlignBottom)
+        return self
+
+    def align_top(self):
+        self.setAlignment(Qt.AlignTop)
+        return self
+
+    def set_font(self, font):
+        self.setFont(font)
+        return self
 
     def set_text(self, text):
         self.set_template(text, text)
@@ -1221,6 +1231,7 @@ class TextLabel(QtWidgets.QLabel):
         self._display = text
         self._text = text
         self._template = template
+        self._saved_selection = None
         self.update_text(self.width())
         self.setText(self._display)
 
@@ -1229,8 +1240,21 @@ class TextLabel(QtWidgets.QLabel):
         if not self._elide:
             return
         text = self._metrics.elidedText(self._template, Qt.ElideRight, width - 2)
-        if text != self._template:
+        if text == self._template:
+            self.setTextFormat(self._text_format)
+        else:
             self._display = text
+            self.setTextFormat(Qt.PlainText)
+
+    def get(self):
+        """Return the label's inner text value"""
+        return self._text
+
+    def start_selection_timer(self):
+        """Start the copy-on-click text selection timer"""
+        if not self.selectedText() and not self.timer.isActive():
+            self.select_all()
+            self.timer.start()
 
     # Qt overrides
     def setFont(self, font):
@@ -1244,18 +1268,47 @@ class TextLabel(QtWidgets.QLabel):
                 self.setText(self._display)
         QtWidgets.QLabel.resizeEvent(self, event)
 
+    def context_menu_actions(self, menu):
+        """Add context menu actions to a QMenu or widget"""
+        self.copy_selection_action.setEnabled(bool(self.selectedText()))
+        menu.addAction(self.copy_all_action)
+        menu.addAction(self.copy_selection_action)
+        menu.addSeparator()
+        menu.addAction(self.select_all_action)
+
+    def _context_menu(self, pos):
+        """Display the custom context menu"""
+        menu = qtutils.create_menu(N_('Actions'), self)
+        self.context_menu_actions(menu)
+        menu.exec_(self.mapToGlobal(pos))
+
+    def mousePressEvent(self, event):
+        self._saved_selection = self.selectedText()
+        super().mouseReleaseEvent(event)
+
+    def mouseReleaseEvent(self, event):
+        """Copy the text label when clicked"""
+        """Copy text when clicked"""
+        # This makes it impossible to select text by clicking and dragging while still
+        # allowing copy-on-click to be a one-click affair.
+        if (
+            self._copy_on_click
+            and event.button() == Qt.LeftButton
+            and not self.selectedText()
+            and not self._saved_selection
+        ):
+            self.copy_all_callback()
+        return super().mouseReleaseEvent(event)
+
 
 class PlainTextLabel(TextLabel):
     """A plaintext label that elides its display"""
 
-    def __init__(self, parent=None):
-        super().__init__(parent=parent, open_external_links=False)
-        self.setTextFormat(Qt.PlainText)
-
-
-class RichTextLabel(TextLabel):
-    """A richtext label that elides its display"""
-
-    def __init__(self, parent=None):
-        super().__init__(parent=parent, open_external_links=True)
-        self.setTextFormat(Qt.RichText)
+    def __init__(self, copy_on_click=True, selectable=True, parent=None):
+        super().__init__(
+            copy_on_click=copy_on_click,
+            selectable=selectable,
+            parent=parent,
+            open_external_links=False,
+            text_format=Qt.PlainText,
+        )
